@@ -27,17 +27,17 @@ Some usage examples:
   $ qsv luau map c "tonumber(a) + tonumber(b)"
 
   Add running total column for Amount
-  $ qsv luau map Total -x "tot = (tot or 0) + Amount; return tot"
+  $ qsv luau map Total "tot = (tot or 0) + Amount; return tot"
 
   Or use the --begin and --end options to compute the running & grand totals
-  $ qsv luau map Total --begin "tot = 0; gtotal = 0" -x \
+  $ qsv luau map Total --begin "tot = 0; gtotal = 0" \
         "tot = tot + Amount; gtotal = gtotal + tot; return tot" --end "return gtotal"
 
   Add running total column for Amount when previous balance was 900
-  $ qsv luau map Total -x "tot = (tot or 900) + Amount; return tot"
+  $ qsv luau map Total "tot = (tot or 900) + Amount; return tot"
 
   Convert Amount to always-positive AbsAmount and Type (debit/credit) columns
-  $ qsv luau map Type -x \
+  $ qsv luau map Type \
         "if tonumber(Amount) < 0 then return 'debit' else return 'credit' end" | \
     qsv luau map AbsAmount "math.abs(tonumber(Amount))"
 
@@ -48,9 +48,15 @@ Some usage examples:
   $ qsv luau filter "tonumber(a) > 45"
   $ qsv luau filter "tonumber(a) >= tonumber(b)"
 
-  Typing long scripts on the command line gets tiresome rather quickly, so use the
-  "file:" prefix to read non-trivial scripts from the filesystem.
-  $ qsv luau map Type -B "file:init.luau" -x "file:debitcredit.luau" -E "file:end.luau"
+  Typing long scripts on the command line gets tiresome rather quickly. Use the
+  "file:" prefix or the ".lua/.luau" file extension to read non-trivial scripts 
+  from the filesystem.
+
+  In the following example, both the BEGIN and END scripts have the lua/luau file extension
+  so they are read from the filesystem.  With the debitcredit.script file, we use the
+  "file:" prefix to read it from the filesystem.
+
+    $ qsv luau map Type -B init.lua file:debitcredit.script -E end.luau
 
 With "luau map", if the MAIN script is invalid for a row, "<ERROR>" followed by a 
 detailed error message is returned for that row.
@@ -131,6 +137,11 @@ Luau arguments:
     The BEGIN script is embedded in the MAIN script by adding a BEGIN block at the top of the script.
     The BEGIN block must start at the beggining of the line. It can contain multiple statements.
 
+    The MAIN script is the main Luau script to execute. It can contain multiple statements and should
+    end with a "return" statement. In map mode, the return value is/are the new value/s of the mapped
+    column/s. In filter mode, the return value is a boolean indicating whether the row should be
+    filtered or not.
+
     The END script is embedded in the MAIN script by adding an END block at the bottom of the script.
     The END block must start at the beginning of the line. It can contain multiple statements.
 
@@ -138,11 +149,6 @@ Luau arguments:
     "luau map". Note that the new columns are added to the CSV after the existing columns.
 
 Luau options:
-    -x, --exec               exec[ute] Luau script, instead of the default eval[uate].
-                             eval (default) expects just a single Luau expression,
-                             while exec expects one or more statements, allowing
-                             full-fledged Luau programs. This only applies to the main-script
-                             argument, not the BEGIN & END scripts.
     -g, --no-globals         Don't create Luau global variables for each column, only col.
                              Useful when some column names mask standard Luau globals.
                              Note: access to Luau globals thru _G remains even without -g.
@@ -181,6 +187,7 @@ Luau options:
                              If the QSV_CKAN_TOKEN envvar is set, it will be used instead.
     --cache-dir <dir>        The directory to use for caching downloaded lookup_table
                              resources using the qsv_register_lookup() helper function.
+                             If the directory does not exist, qsv will attempt to create it.
                              If the QSV_CACHE_DIR envvar is set, it will be used instead.
                              [default: qsv-cache]
 
@@ -230,7 +237,6 @@ struct Args {
     arg_new_columns:  Option<String>,
     arg_main_script:  String,
     arg_input:        Option<String>,
-    flag_exec:        bool,
     flag_no_globals:  bool,
     flag_remap:       bool,
     flag_begin:       Option<String>,
@@ -345,7 +351,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     }
     luau_script = main_script;
 
-    let mut main_script = if args.flag_exec {
+    // if the main script is a single expression, we need to prepend a return statement
+    let mut main_script = if luau_script.contains("return") {
         String::new()
     } else {
         String::from("return ")
@@ -708,7 +715,7 @@ fn sequential_mode(
                 let err_msg = format!("<ERROR> _IDX: {idx} error({error_count}): {e:?}");
                 log::error!("{err_msg}");
 
-                mlua::ToLua::to_lua(err_msg, luau)
+                mlua::IntoLua::into_lua(err_msg, luau)
                     .map_err(|e| format!("Failed to convert error message to Lua: {e}"))?
             }
         };
@@ -802,7 +809,7 @@ fn sequential_mode(
                 log::error!("{err_msg}");
                 log::error!("END globals: {globals:?}");
 
-                mlua::ToLua::to_lua(err_msg, luau)
+                mlua::IntoLua::into_lua(err_msg, luau)
                     .map_err(|e| format!("Failed to convert error message to Lua: {e}"))?
             }
         };
@@ -1044,7 +1051,7 @@ fn random_acess_mode(
                 let err_msg = format!("<ERROR> _IDX: {curr_record} error({error_count}): {e:?}");
                 log::error!("{err_msg}");
 
-                mlua::ToLua::to_lua(err_msg, luau)
+                mlua::IntoLua::into_lua(err_msg, luau)
                     .map_err(|e| format!("Failed to convert error message to Lua: {e}"))?
             }
         };
@@ -1141,7 +1148,7 @@ fn random_acess_mode(
                 log::error!("{err_msg}");
                 log::error!("END globals: {globals:?}");
 
-                mlua::ToLua::to_lua(err_msg, luau)
+                mlua::IntoLua::into_lua(err_msg, luau)
                     .map_err(|e| format!("Failed to convert error message to Lua: {e}"))?
             }
         };
@@ -1356,6 +1363,16 @@ fn setup_helpers(
     ckan_api_url: String,
     ckan_token: Option<String>,
 ) -> Result<(), CliError> {
+    macro_rules! helper_err {
+        ($helper_name:literal, $($arg:tt)*) => ({
+            use log::error;
+            let helper_name = format!("{}: ", $helper_name);
+            let err_msg = format!($($arg)*);
+            error!("{helper_name}: {err_msg}");
+            Err(mlua::Error::RuntimeError(err_msg))
+        });
+    }
+
     // this is a helper function that can be called from Luau scripts
     // to send log messages to the logfile
     // the first parameter is the log level, and the following parameters are concatenated
@@ -1436,9 +1453,10 @@ fn setup_helpers(
     //
     let qsv_break = luau.create_function(|luau, mut args: mlua::MultiValue| {
         if LUAU_STAGE.load(Ordering::Relaxed) == Stage::End as i8 {
-            return Err(mlua::Error::RuntimeError(
-                "qsv_break() can only be called from the BEGIN and MAIN scripts.".to_string(),
-            ));
+            return helper_err!(
+                "qsv_break",
+                "qsv_break() can only be called from the BEGIN and MAIN scripts."
+            );
         }
 
         let mut break_msg = String::new();
@@ -1486,9 +1504,10 @@ fn setup_helpers(
     //
     let qsv_skip = luau.create_function(|_, ()| {
         if LUAU_STAGE.load(Ordering::Relaxed) != Stage::Main as i8 {
-            return Err(mlua::Error::RuntimeError(
-                "qsv_skip() can only be called from the MAIN script.".to_string(),
-            ));
+            return helper_err!(
+                "qsv_skip",
+                "qsv_skip() can only be called from the MAIN script."
+            );
         }
 
         QSV_SKIP.store(true, Ordering::Relaxed);
@@ -1512,9 +1531,10 @@ fn setup_helpers(
     //
     let qsv_autoindex = luau.create_function(|_, ()| {
         if LUAU_STAGE.load(Ordering::Relaxed) != Stage::Begin as i8 {
-            return Err(mlua::Error::RuntimeError(
-                "qsv_autoindex() can only be called from the BEGIN script.".to_string(),
-            ));
+            return helper_err!(
+                "qsv_autoindex",
+                "qsv_autoindex() can only be called from the BEGIN script."
+            );
         }
 
         Ok(())
@@ -1534,9 +1554,7 @@ fn setup_helpers(
     //
     let qsv_setenv = luau.create_function(|_, (envvar, value): (String, String)| {
         if envvar.is_empty() {
-            return Err(mlua::Error::RuntimeError(
-                "qsv_setenv() - envvar cannot be empty.".to_string(),
-            ));
+            return helper_err!("qsv_setenv", "envvar cannot be empty.");
         }
 
         if value.is_empty() {
@@ -1560,9 +1578,7 @@ fn setup_helpers(
     //
     let qsv_getenv = luau.create_function(|_, envvar: String| {
         if envvar.is_empty() {
-            return Err(mlua::Error::RuntimeError(
-                "qsv_getenv() - envvar cannot be empty.".to_string(),
-            ));
+            return helper_err!("qsv_getenv", "envvar cannot be empty.");
         }
 
         match std::env::var(envvar) {
@@ -1572,6 +1588,23 @@ fn setup_helpers(
     })?;
     luau.globals().set("qsv_getenv", qsv_getenv)?;
 
+    // this is a helper function to check if a file exists.
+    //
+    //   qsv_fileexists(filepath: string)
+    //        filepath: the path to the file to check
+    //         returns: true if the file exists, false otherwise.
+    //                  A Luau runtime error if the filepath argument is empty.
+    //
+    let qsv_fileexists = luau.create_function(|_, filepath: String| {
+        if filepath.is_empty() {
+            return helper_err!("qsv_fileexists", "filepath cannot be empty.");
+        }
+
+        let path = Path::new(&filepath);
+        Ok(path.exists())
+    })?;
+    luau.globals().set("qsv_fileexists", qsv_fileexists)?;
+
     // this is a helper function that can be called from the BEGIN, MAIN & END scripts to write to
     // a file. The file will be created if it does not exist. The file will be appended to if it
     // already exists. The filename will be sanitized and will be written to the current working
@@ -1725,10 +1758,7 @@ fn setup_helpers(
                 Ok(output_table)
             }
             Err(e) => {
-                log::error!("failed to execute qsv command: {e}");
-                Err(mlua::Error::RuntimeError(format!(
-                    r#"failed to execute qsv command "{qsv_args}": {e}"#
-                )))
+                helper_err!("qsv_cmd", "failed to execute qsv command: {qsv_args}: {e}")
             }
         }
     })?;
@@ -1780,12 +1810,10 @@ fn setup_helpers(
 
         let shellcmd_string = shellcmd.to_ascii_lowercase();
         let Ok(_) = ShellCmd::from_str(&shellcmd_string) else {
-            return Err(mlua::Error::RuntimeError(format!(
-                "Invalid shell command: \"{shellcmd}\". \
+            return helper_err!("qsv_shellcmd", "Invalid shell command: \"{shellcmd}\". \
                 Only the following commands are allowed: \
                 awk, cat, cp, cut, df, echo, rg, grep, head, ls, mkdir, \
-                mv, nl, pwd, sed, sort, tail, touch, tr, uname, uniq, wc, whoami"
-            )))
+                mv, nl, pwd, sed, sort, tail, touch, tr, uname, uniq, wc, whoami")
         };
 
         let mut cmd = std::process::Command::new(shellcmd_string.clone());
@@ -1822,11 +1850,10 @@ fn setup_helpers(
                 Ok(output_table)
             }
             Err(e) => {
-                let err_msg = format!(
-                    r#"failed to execute shell command "{shellcmd_string}" "{args_string}": {e}"#
-                );
-                log::error!("{err_msg}");
-                Err(mlua::Error::RuntimeError(err_msg))
+                helper_err!(
+                    "qsv_shellcmd",
+                    "failed to execute shell command: {shellcmd_string} {args_string}: {e}"
+                )
             }
         }
     })?;
@@ -1870,17 +1897,8 @@ fn setup_helpers(
     let qsv_register_lookup = luau.create_function(move |luau, (lookup_name, mut lookup_table_uri, cache_age_secs): (String, String, i64)| {
         const MSG_PREFIX: &str = "qsv_register_lookup() - ";
 
-        macro_rules! lookup_err {
-            ($($arg:tt)*) => ({
-                use log::error;
-                let err_msg = format!($($arg)*);
-                error!("{MSG_PREFIX}{err_msg}");
-                Err(mlua::Error::RuntimeError(err_msg))
-            });
-        }
-
         if LUAU_STAGE.load(Ordering::Relaxed) != Stage::Begin as i8 {
-            return lookup_err!("can only be called from the BEGIN script.");
+            return helper_err!("qsv_register_lookup", "can only be called from the BEGIN script.");
         }
 
         let call_parameters = format!("qsv_lookup_register({lookup_name}, {lookup_table_uri}, {cache_age_secs})");
@@ -1971,7 +1989,7 @@ fn setup_helpers(
                 {
                     Ok(c) => c,
                     Err(e) => {
-                        return lookup_err!("Cannot build reqwest client to download lookup CSV: {e}.");
+                        return helper_err!("qsv_register_lookup", "Cannot build reqwest client to download lookup CSV: {e}.");
                     }
                 };
 
@@ -2003,26 +2021,26 @@ fn setup_helpers(
                         let validated_url = match Url::parse(&lookup_table_uri) {
                             Ok(url) => url,
                             Err(e) => {
-                                return lookup_err!("Invalid resource_search url {e}.");
+                                return helper_err!("qsv_register_lookup", "Invalid resource_search url {e}.");
                             }
                         };
 
                         let resource_search_result = match client.get(validated_url).headers(headers.clone()).send() {
                             Ok(response) => response.text().unwrap_or_default(),
                             Err(e) => {
-                                return lookup_err!("Cannot find resource name with resource_search: {e}.");
+                                return helper_err!("qsv_register_lookup", "Cannot find resource name with resource_search: {e}.");
                             }
                         };
 
                         let resource_search_json: serde_json::Value = match serde_json::from_str(&resource_search_result) {
                             Ok(json) => json,
                             Err(e) => {
-                                return lookup_err!("Invalid resource_search json {e}.");
+                                return helper_err!("qsv_register_lookup", "Invalid resource_search json {e}.");
                             }
                         };
 
                         let Some(resource_id) = resource_search_json["result"]["results"][0]["id"].as_str() else {
-                            return lookup_err!("Cannot find a resource name.");
+                            return helper_err!("qsv_register_lookup", "Cannot find a resource name.");
                         };
 
                         lookup_table_uri = format!("{ckan_api_url}/resource_show?id={resource_id}");
@@ -2032,25 +2050,25 @@ fn setup_helpers(
                     let resource_show_result = match client.get(lookup_table_uri).headers(headers.clone()).send() {
                         Ok(response) => response.text().unwrap_or_default(),
                         Err(e) => {
-                            return lookup_err!("CKAN scheme used. Cannot get lookup CSV resource: {e}.");
+                            return helper_err!("qsv_register_lookup", "CKAN scheme used. Cannot get lookup CSV resource: {e}.");
                         }
                     };
 
                     let resource_show_json: serde_json::Value = match serde_json::from_str(&resource_show_result) {
                         Ok(json) => json,
                         Err(e) => {
-                            return lookup_err!("Invalid resource_show json: {e}.");
+                            return helper_err!("qsv_register_lookup", "Invalid resource_show json: {e}.");
                         }
                     };
 
                     let Some(url) = resource_show_json["result"]["url"].as_str() else {
-                        return lookup_err!("Cannot get resource URL from resource_show JSON response.: {resource_show_json}");
+                        return helper_err!("qsv_register_lookup", "Cannot get resource URL from resource_show JSON response.: {resource_show_json}");
                     };
 
                     match client.get(url).headers(headers).send() {
                         Ok(response) => response,
                         Err(e) => {
-                            return lookup_err!(r#"Cannot read lookup CSV at "{url}": {e}."#);
+                            return helper_err!("qsv_register_lookup", r#"Cannot read lookup CSV at "{url}": {e}."#);
                         }
                     }
                 } else {
@@ -2059,7 +2077,7 @@ fn setup_helpers(
                     let validated_url = match Url::parse(&lookup_table_uri) {
                         Ok(url) => url,
                         Err(e) => {
-                            return lookup_err!("Invalid lookup CSV url {e}.");
+                            return helper_err!("qsv_register_lookup", "Invalid lookup CSV url {e}.");
                         }
                     };
 
@@ -2080,7 +2098,7 @@ fn setup_helpers(
                     match client.get(validated_url.clone()).headers(headers).send() {
                         Ok(response) => response,
                         Err(e) => {
-                            return lookup_err!(r#"Cannot read lookup CSV at "{validated_url}": {e}."#);
+                            return helper_err!("qsv_register_lookup", r#"Cannot read lookup CSV at "{validated_url}": {e}."#);
                         }
                     }
                 };
@@ -2098,7 +2116,7 @@ fn setup_helpers(
                         match lookup_csv_response.error_for_status_ref() {
                             Ok(_) => (),
                             Err(e) => {
-                                return lookup_err!("Cannot read lookup CSV at url: {e}.");
+                                return helper_err!("qsv_register_lookup", "Cannot read lookup CSV at url: {e}.");
                             }
                         }
                     }
@@ -2111,7 +2129,7 @@ fn setup_helpers(
                     let mut cache_file = match std::fs::File::create(&cache_file_path) {
                         Ok(f) => f,
                         Err(e) => {
-                            return lookup_err!(
+                            return helper_err!("qsv_register_lookup", 
                                 "Cannot create cache file {}: {e}.", cache_file_path.display());
                         }
                     };
@@ -2143,7 +2161,7 @@ fn setup_helpers(
         let headers = match rdr.headers() {
             Ok(headers) => headers.clone(),
             Err(e) => {
-                return lookup_err!("Cannot read headers of lookup table: {e}");
+                return helper_err!("qsv_register_lookup", "Cannot read headers of lookup table: {e}");
             }
         };
         for result in rdr.records() {
