@@ -1,12 +1,17 @@
 static USAGE: &str = r#"
 Generate JSON Schema from CSV data.
 
-This command generates reference JSON Schema (Draft 7) from CSV data, 
+This command derives a JSON Schema Validation (Draft 7) file from CSV data, 
 including validation rules based on data type and input data domain/range.
+https://json-schema.org/draft/2020-12/json-schema-validation.html
 
 Running `validate` command on original input CSV with generated schema 
 should not flag any invalid records.
 
+The intended workflow is to use `schema` command to generate a schema file from
+representative CSV data, adjust the schema file as needed, and then use `validate`
+command to validate other CSV data using the generated schema.
+
 Generated schema file has `.schema.json` postfix appended. For example, 
 for input `mydata.csv`, schema file would be `mydata.csv.schema.json`.
 
@@ -30,7 +35,11 @@ Schema options:
                                columns are inferred as date/datetime, they are set
                                to type "string" in the schema instead of
                                "date" or "date-time".
-    --pattern-columns <args>   Select columns to add pattern constraints
+    --pattern-columns <args>   Select columns to derive regex pattern constraints.
+                               That is, this will create a regular expression
+                               that matches all values for each specified column.
+                               Columns are selected using `select` syntax 
+                               (see `qsv select --help` for details).
     --dates-whitelist <list>   The case-insensitive patterns to look for when 
                                shortlisting fields for date inference.
                                i.e. if the field's name has any of these patterns,
@@ -48,7 +57,7 @@ Schema options:
 Common options:
     -h, --help                 Display this message
     -n, --no-headers           When set, the first row will not be interpreted
-                               as headers. Namely, it will be sorted with the rest
+                               as headers. Namely, it will be processed with the rest
                                of the rows. Otherwise, the first row will always
                                appear as the header row in the output.
     -d, --delimiter <arg>      The field delimiter for reading CSV data.
@@ -127,6 +136,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     // we can do this directly here, since args is mutable and
     // Config has not been created yet at this point
     args.flag_prefer_dmy = args.flag_prefer_dmy || std::env::var("QSV_PREFER_DMY").is_ok();
+    if args.flag_prefer_dmy {
+        winfo!("Prefer DMY set.");
+    }
 
     // build schema for each field by their inferred type, min/max value/length, and unique values
     let mut properties_map: Map<String, Value> =
@@ -146,7 +158,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         if pattern_map.contains_key(field_name) && should_emit_pattern_constraint(field_def) {
             let field_def_map = field_def.as_object_mut().unwrap();
             let pattern = Value::String(pattern_map[field_name].clone());
-            field_def_map.insert("pattern".to_string(), pattern);
+            field_def_map.insert("pattern".to_string(), pattern.clone());
+            winfo!("Added regex pattern constraint for field: {field_name} -> {pattern}");
         }
     }
 
@@ -372,6 +385,10 @@ pub fn infer_schema_from_stats(args: &Args, input_filename: &str) -> CliResult<M
 
         if !enum_list.is_empty() {
             field_map.insert("enum".to_string(), Value::Array(enum_list.clone()));
+            winfo!(
+                "Enum list generated for field '{header_string}' ({} value/s)",
+                enum_list.len()
+            );
         }
 
         // add current field definition to properties map
@@ -409,11 +426,14 @@ fn get_stats_records(args: &Args) -> CliResult<(ByteRecord, Vec<Stats>, AHashMap
     let (csv_fields, csv_stats) = match stats_args.rconfig().indexed() {
         Ok(o) => {
             if let Some(idx) = o {
-                info!("has index, triggering parallel stats");
+                winfo!("index found... triggering parallel stats");
                 let idx_count = idx.count();
                 stats_args.parallel_stats(&stats_args.flag_dates_whitelist, idx_count)
             } else {
-                info!("no index, triggering sequential stats");
+                winfo!(
+                    "no index, triggering sequential stats. Consider indexing your data if schema \
+                     inferencing is slow. "
+                );
                 stats_args.sequential_stats(&stats_args.flag_dates_whitelist)
             }
         }
@@ -633,8 +653,6 @@ fn generate_string_patterns(
 
         // build regex based on unique values
         let regexp: String = RegExpBuilder::from(&values)
-            .with_conversion_of_digits()
-            .with_conversion_of_words()
             .with_conversion_of_repetitions()
             .with_minimum_repetitions(2)
             .build();
