@@ -1,14 +1,17 @@
 static USAGE: &str = r#"
-Validate CSV data with JSON Schema, and put invalid records into a separate file.
+Validate CSV data with JSON Schema Validation, putting invalid records into a separate file.
+Uses https://json-schema.org/draft/2020-12/json-schema-validation.html.
 When run without JSON Schema, only a simple CSV check (RFC 4180) is performed.
 
+You can create a JSON Schema from a CSV file using the `qsv schema` command.
+
 Example output files from `mydata.csv`. If piped from stdin, then filename is `stdin.csv`.
 
 * mydata.csv.valid
 * mydata.csv.invalid
 * mydata.csv.validation-errors.tsv
 
-JSON Schema can be a local file or a URL.
+The JSON Schema can be a local file or a URL.
 
 Returns exitcode 0 when the CSV file is valid, exitcode > 0 otherwise.
 If all records are valid, no output files are produced.
@@ -146,7 +149,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         progress.set_draw_target(ProgressDrawTarget::hidden());
     }
 
-    // if no json schema supplied, only let csv reader RFC4180-validate csv file
+    // if no JSON Schema supplied, only let csv reader RFC4180-validate csv file
     if args.arg_json_schema.is_none() {
         // just read csv file and let csv reader report problems
 
@@ -779,7 +782,7 @@ mod tests_for_csv_to_json_conversion {
     }
 }
 
-/// Validate JSON instance against compiled JSON schema
+/// Validate JSON instance against compiled JSON Schema
 /// If invalid, returns Some(Vec<(String,String)>) holding the error messages
 #[inline]
 fn validate_json_instance(
