@@ -84,6 +84,7 @@ pub struct Config {
     trim:              csv::Trim,
     autoindex:         bool,
     prefer_dmy:        bool,
+    comment:           Option<u8>,
 }
 
 // Empty trait as an alias for Seek and Read that avoids auto trait errors
@@ -160,6 +161,7 @@ impl Config {
             trim: csv::Trim::None,
             autoindex: env::var("QSV_AUTOINDEX").is_ok(),
             prefer_dmy: env::var("QSV_PREFER_DMY").is_ok(),
+            comment: None,
         }
     }
 
@@ -174,6 +176,11 @@ impl Config {
         self.delimiter
     }
 
+    pub const fn comment(mut self, c: Option<u8>) -> Config {
+        self.comment = c;
+        self
+    }
+
     pub const fn get_dmy_preference(&self) -> bool {
         self.prefer_dmy
     }
@@ -306,8 +313,6 @@ impl Config {
     }
 
     fn autoindex_file(&self) {
-        use io::prelude::*;
-
         // autoindex_file should never panic. It should silently fail as its a "convenience fn"
         // that's why we have a lot of let-else returns, in lieu of unwraps
         let Some(path_buf) = &self.path else { return };
@@ -318,7 +323,7 @@ impl Config {
         let mut wtr = io::BufWriter::new(idxfile);
         match csv_index::RandomAccessSimple::create(&mut rdr, &mut wtr) {
             Ok(_) => {
-                let Ok(_) = wtr.flush() else { return };
+                let Ok(_) = io::Write::flush(&mut wtr) else { return };
                 debug!("autoindex of {path_buf:?} successful.");
             }
             Err(e) => debug!("autoindex of {path_buf:?} failed: {e}"),
@@ -424,9 +429,11 @@ impl Config {
             .unwrap_or_else(|_| DEFAULT_RDR_BUFFER_CAPACITY.to_string());
         let rdr_buffer: usize = rdr_capacitys.parse().unwrap_or(DEFAULT_RDR_BUFFER_CAPACITY);
 
-        let rdr_comment: Option<u8> = env::var("QSV_COMMENT_CHAR")
-            .ok()
-            .map(|s| s.as_bytes().first().unwrap().to_owned());
+        let rdr_comment: Option<u8> = if let Ok(comment_char) = env::var("QSV_COMMENT_CHAR") {
+            Some(comment_char.as_bytes().first().unwrap().to_owned())
+        } else {
+            self.comment
+        };
 
         csv::ReaderBuilder::new()
             .flexible(self.flexible)
