@@ -36,6 +36,7 @@
 extern crate crossbeam_channel as channel;
 use std::{env, io, time::Instant};
 
+extern crate qsv_docopt as docopt;
 use docopt::Docopt;
 use serde::Deserialize;
 
@@ -63,6 +64,7 @@ macro_rules! command_list {
     help        Show this usage message
     index       Create CSV index for faster access
     input       Read CSVs w/ special quoting, skipping, trimming & transcoding rules
+    luau        Execute Luau script on CSV data
     pseudo      Pseudonymise the values of a column
     rename      Rename the columns of CSV data efficiently
     replace     Replace patterns in CSV data
@@ -210,8 +212,8 @@ enum Command {
     Help,
     Index,
     Input,
-    // #[cfg(feature = "luau")]
-    // Luau,
+    #[cfg(feature = "luau")]
+    Luau,
     Pseudo,
     Rename,
     Replace,
@@ -256,8 +258,8 @@ impl Command {
             }
             Command::Index => cmd::index::run(argv),
             Command::Input => cmd::input::run(argv),
-            // #[cfg(feature = "luau")]
-            // Command::Luau => cmd::luau::run(argv),
+            #[cfg(feature = "luau")]
+            Command::Luau => cmd::luau::run(argv),
             Command::Pseudo => cmd::pseudo::run(argv),
             Command::Rename => cmd::rename::run(argv),
             Command::Replace => cmd::replace::run(argv),
